@@ -21,7 +21,7 @@ import cffi
 import numpy as np
 
 
-__version__ = "1.6.0"
+__version__ = "1.7.0"
 
 
 def string_from_ffi(s):
@@ -378,11 +378,11 @@ def push_vector(npa):
     elif dtype == np.float32:  #  can directly pass the data buffer
         cffi_buffer = ffi.cast("float*", npa.ctypes.data)
         lib.p_push_vector_from_float32_array(cffi_buffer, len(npa), np.nan)
-    elif dtype == np.bool:  # convert first to float32
+    elif dtype == bool:  # convert first to float32
         f32_array = npa.astype(np.float32)
         cffi_buffer = ffi.cast("float*", f32_array.ctypes.data)
         lib.p_push_vector_from_float32_array(cffi_buffer, len(f32_array), np.nan)
-    elif dtype == np.int:  # convert first to float64
+    elif dtype == int:  # convert first to float64
         f64_array = npa.astype(np.float64)
         cffi_buffer = ffi.cast("double*", f64_array.ctypes.data)
         lib.p_push_vector_from_double_array(cffi_buffer, len(f64_array), np.nan)
@@ -538,12 +538,15 @@ class ContainerValue(Value):
             elif isinstance(index, np.ndarray):  # can have an array as an index
                 return subset(self, index + self.macro_index_base)
             else:
+                c = int(count(self))
                 if index < 0:  # negative index valid for range [-len..-1]
-                    c = int(count(self))
                     if index >= -c:
                         index = c + index
                     else:
                         raise IndexError("Index " + str(index) + " invalid ", self)
+                else:
+                    if index > c - 1:
+                        raise IndexError("Index " + str(index) + " invalid ", self)
                 return subset(
                     self, index + self.macro_index_base
                 )  # numeric index: 0->1-based
@@ -586,20 +589,18 @@ class Fieldset(FileBackedValueWithOperators, ContainerValue):
 
     def append(self, other):
         temp = merge(self, other)
-        if self.val_pointer != None:  #  we will overwrite ourselves, so delete
+        if self.val_pointer is not None:  #  we will overwrite ourselves, so delete
             lib.p_destroy_value(self.val_pointer)
         self.steal_val_pointer(temp)
 
-    def to_dataset(self):
+    def to_dataset(self, **kwarg):
         # soft dependency on cfgrib
         try:
-            from cfgrib import xarray_store
+            import xarray as xr
         except ImportError:
-            print(
-                "Package cfgrib/xarray_store not found. Try running 'pip install cfgrib'."
-            )
+            print("Package xarray not found. Try running 'pip install xarray'.")
             raise
-        dataset = xarray_store.open_dataset(self.url())
+        dataset = xr.open_dataset(self.url(), engine="cfgrib", backend_kwargs=kwarg)
         return dataset
 
     def __getstate__(self):
@@ -1015,8 +1016,8 @@ def bind_functions(namespace, module_name=None):
     namespace["setoutput"] = setoutput
     namespace["metzoom"] = metzoom
     namespace["version_info"] = version_info
+    namespace["merge"] = merge
     namespace["dataset_to_fieldset"] = dataset_to_fieldset
-    namespace["download_gallery_data"] = download_gallery_data
 
     namespace["Fieldset"] = Fieldset
     namespace["Request"] = Request
@@ -1034,7 +1035,7 @@ greater_equal_than = make(">=")
 greater_than = make(">")
 lower_equal_than = make("<=")
 lower_than = make("<")
-merge = make("&")
+met_merge = make("&")
 met_not_eq = make("<>")
 met_plot = make("plot")
 mod = make("mod")
@@ -1059,10 +1060,18 @@ write = make("write")
 # Python bindings to the resulting dict
 def version_info():
     binary_info = dict(met_version_info())
-    binary_info['metview_python_version'] = __version__
+    binary_info["metview_python_version"] = __version__
     return binary_info
 
 
+# wrapper so that we can merge a single value (just returns itself)
+def merge(*args):
+    if len(args) == 1:
+        return args[0]
+    else:
+        return met_merge(*args)
+
+
 # -----------------------------------------------------------------------------
 #                        Particular code for calling the plot() command
 # -----------------------------------------------------------------------------
@@ -1074,16 +1083,20 @@ class Plot:
         self.jupyter_args = {}
 
     def __call__(self, *args, **kwargs):
+        # if animate=True is supplied, then create a Jupyter animation
+        if kwargs.get("animate", False):
+            return animate(args, kwargs)
+
+        # otherwise create a single static plot
         if self.plot_to_jupyter:
             f, tmp = tempfile.mkstemp(".png")
             os.close(f)
 
             base, ext = os.path.splitext(tmp)
 
-            self.jupyter_args.update(output_name=base, output_name_first_page_number="off")
-            met_setoutput(
-                png_output(self.jupyter_args)
-            )
+            output_args = {"output_name": base, "output_name_first_page_number": "off"}
+            output_args.update(self.jupyter_args)
+            met_setoutput(png_output(output_args))
             met_plot(*args)
 
             image = Image(tmp)
@@ -1107,6 +1120,78 @@ class Plot:
 
 plot = Plot()
 
+# animate - only usable within Jupyter notebooks
+# generates a widget allowing the user to select between plot frames
+def animate(*args, **kwargs):
+
+    if not plot.plot_to_jupyter:
+        raise EnvironmentError(
+            "animate() can only be used after calling set_output('jupyter')"
+        )
+
+    import ipywidgets as widgets
+
+    # create all the widgets first so that the 'waiting' label is at the bottom
+    image_widget = widgets.Image(
+        format="png"
+        # width=300,
+        # height=400,
+    )
+
+    frame_widget = widgets.IntSlider(
+        value=1,
+        min=1,
+        max=1,
+        step=1,
+        description="Frame:",
+        disabled=False,
+        continuous_update=True,
+        readout=True,
+    )
+
+    image_widget.layout.visibility = "hidden"
+    frame_widget.layout.visibility = "hidden"
+    waitl_widget = widgets.Label(value="Generating plots....")
+    frame_widget.layout.width = "800px"
+    display(image_widget, frame_widget, waitl_widget)
+
+    # plot all frames to a temporary directory owned by Metview to enure cleanup
+    tempdirpath = tempfile.mkdtemp(dir=os.environ.get("METVIEW_TMPDIR", None))
+    plot_path = os.path.join(tempdirpath, "plot")
+    met_setoutput(
+        png_output(
+            output_name=plot_path, output_file_minimal_width=3, **plot.jupyter_args
+        )
+    )
+    met_plot(*args)
+    (_, _, filenames) = next(os.walk(tempdirpath), (None, None, None))
+
+    if filenames is None:
+        waitl_widget.value = "No plots generated"
+        return
+
+    files = [os.path.join(tempdirpath, f) for f in sorted(filenames)]
+    frame_widget.max = len(files)
+    frame_widget.description = "Frame (" + str(len(files)) + ") :"
+
+    def plot_frame(frame_index):
+        im_file = open(files[frame_index - 1], "rb")
+        imf = im_file.read()
+        im_file.close()
+        image_widget.value = imf
+
+    def on_frame_change(change):
+        plot_frame(change["new"])
+
+    plot_frame(1)
+    frame_widget.observe(on_frame_change, names="value")
+
+    # everything is ready now, so hide the 'waiting' label
+    # and reveal the plot and the frame slider
+    waitl_widget.layout.visibility = "hidden"
+    image_widget.layout.visibility = "visible"
+    frame_widget.layout.visibility = "visible"
+
 
 # On a test system, importing IPython took approx 0.5 seconds, so to avoid that hit
 # under most circumstances, we only import it when the user asks for Jupyter
@@ -1136,16 +1221,3 @@ def setoutput(*args, **kwargs):
     else:
         plot.plot_to_jupyter = False
         met_setoutput(*args)
-
-
-# -----------------------------------------------------------------------------
-#                        Other user-visible utiliy functions
-# -----------------------------------------------------------------------------
-
-def download_gallery_data(filename):
-    base_url = "http://download.ecmwf.org/test-data/metview/gallery/"
-    try:
-        d = download(url=base_url + filename, target=filename)
-        return d
-    except:
-        raise Exception("Could not download file " + filename + " from the download server")
